@@ -3,7 +3,11 @@
 Translates pygls types back and forth with Jedi
 """
 
+import functools
+import inspect
 import sys
+import threading
+from ast import PyCF_ONLY_AST
 from inspect import Parameter
 from typing import Dict, Iterator, List, Optional, Tuple
 
@@ -13,7 +17,7 @@ import jedi.inference.references
 import jedi.settings
 from jedi import Project, Script
 from jedi.api.classes import BaseName, Completion, Name, ParamName, Signature
-from pygls.lsp.types import (
+from lsprotocol.types import (
     CompletionItem,
     CompletionItemKind,
     Diagnostic,
@@ -34,6 +38,42 @@ from .initialization_options import HoverDisableOptions, InitializationOptions
 from .type_map import get_lsp_completion_type, get_lsp_symbol_type
 
 
+def debounce(interval_s, keyed_by=None):
+    """Debounce calls to this function until interval_s seconds have passed.
+
+    Decorator copied from https://github.com/python-lsp/python-lsp-
+    server
+    """
+
+    def wrapper(func):
+        timers = {}
+        lock = threading.Lock()
+
+        @functools.wraps(func)
+        def debounced(*args, **kwargs):
+            sig = inspect.signature(func)
+            call_args = sig.bind(*args, **kwargs)
+            key = call_args.arguments[keyed_by] if keyed_by else None
+
+            def run():
+                with lock:
+                    del timers[key]
+                return func(*args, **kwargs)
+
+            with lock:
+                old_timer = timers.get(key)
+                if old_timer:
+                    old_timer.cancel()
+
+                timer = threading.Timer(interval_s, run)
+                timers[key] = timer
+                timer.start()
+
+        return debounced
+
+    return wrapper
+
+
 def _jedi_debug_function(
     color: str,  # pylint: disable=unused-argument
     str_out: str,
@@ -150,14 +190,23 @@ def lsp_document_symbols(names: List[Name]) -> List[DocumentSymbol]:
     accessible with dot notation are removed from display. See comments
     inline for cleaning steps.
     """
+    # pylint: disable=too-many-branches
     _name_lookup: Dict[Name, DocumentSymbol] = {}
     results: List[DocumentSymbol] = []
     for name in names:
+        symbol_range = _document_symbol_range(name)
+        if symbol_range is None:
+            continue
+
+        selection_range = lsp_range(name)
+        if selection_range is None:
+            continue
+
         symbol = DocumentSymbol(
             name=name.name,
             kind=get_lsp_symbol_type(name.type),
-            range=_document_symbol_range(name),
-            selection_range=lsp_range(name),
+            range=symbol_range,
+            selection_range=selection_range,
             detail=name.description,
             children=[],
         )
@@ -184,14 +233,19 @@ def lsp_document_symbols(names: List[Name]) -> List[DocumentSymbol]:
         elif parent not in _name_lookup:
             # unqualified names are not included in the tree
             continue
-        elif name.is_side_effect() and name.get_line_code().strip().startswith(
-            "self."
+        elif (
+            name.is_side_effect()
+            and parent.name == "__init__"
+            and name.get_line_code().strip().startswith("self.")
         ):
             # handle attribute creation on __init__ method
             symbol.kind = SymbolKind.Property
-            parent_symbol = _name_lookup[parent]
-            assert parent_symbol.children is not None
-            parent_symbol.children.append(symbol)
+            grandparent_symbol = _name_lookup.get(parent.parent())
+            if grandparent_symbol is not None and (
+                grandparent_symbol.kind == SymbolKind.Class
+            ):
+                assert grandparent_symbol.children is not None
+                grandparent_symbol.children.append(symbol)
         elif parent.type == "class":
             # children are added for class scopes
             if name.type == "function":
@@ -229,6 +283,38 @@ def lsp_diagnostic(error: jedi.api.errors.SyntaxError) -> Diagnostic:
     )
 
 
+def lsp_python_diagnostic(uri: str, source: str) -> Optional[Diagnostic]:
+    """Get LSP Diagnostic using the compile builtin."""
+    try:
+        compile(source, uri, "exec", PyCF_ONLY_AST)
+        return None
+    except SyntaxError as err:
+        column = err.offset - 1 if err.offset is not None else 0
+        line = err.lineno - 1 if err.lineno is not None else 0
+
+        _until_column = getattr(err, "end_offset", None)
+        _until_line = getattr(err, "end_lineno", None)
+
+        until_column = (
+            _until_column - 1 if _until_column is not None else column + 1
+        )
+        until_line = _until_line - 1 if _until_line is not None else line + 1
+
+        if (line, column) >= (until_line, until_column):
+            until_column, until_line = column, line
+            column = 0
+
+        return Diagnostic(
+            range=Range(
+                start=Position(line=line, character=column),
+                end=Position(line=until_line, character=until_column),
+            ),
+            message=err.__class__.__name__ + ": " + str(err),
+            severity=DiagnosticSeverity.Error,
+            source="compile",
+        )
+
+
 def line_column(position: Position) -> Tuple[int, int]:
     """Translate pygls Position to Jedi's line/column.
 
